=== modified file 'NEWS'
--- NEWS	2008-10-28 17:40:04 +0000
+++ NEWS	2008-10-28 17:41:35 +0000
@@ -11,11 +11,23 @@
 
   NEW FEATURES:
 
+    * New Branch hook ``transform_fallback_location`` allows a function to
+      be called when looking up the stacked source. (Michael Hudson)
+
   IMPROVEMENTS:
 
     * ``bzr dump-btree`` is a hidden command introduced to allow dumping
       the contents of a compressed btree file.  (John Arbash Meinel)
 
+    * ``bzr pack`` now tells the index builders to optimize for size. For
+      btree index repositories, this can save 25% of the index size
+      (mostly in the text indexes). (John Arbash Meinel)
+
+    * ``bzr push`` to an existing branch or repository on a smart server
+      is faster, due to Bazaar making more use of the ``get_parent_map``
+      RPC when querying the remote branch's revision graph.
+      (Andrew Bennetts)
+
     * default username for bzr+ssh and sftp can be configured in
       authentication.conf. (Aaron Bentley)
 
@@ -37,6 +49,10 @@
       memory before writing it to disk. This was a little bit faster, but
       consumed lots of memory. (John Arbash Meinel, #269456)
 
+    * ``bzr missing --quiet`` no longer prints messages about whether
+      there are missing revisions.  The exit code indicates whether there
+      were or not.  (Martin Pool, #284748)
+
     * Fixes to the ``annotate`` code. The fast-path which re-used the
       stored deltas was accidentally disabled all the time, instead of
       only when a branch was stacked. Second, the code would accidentally
@@ -44,6 +60,9 @@
       could only happen if ``bzr reconcile`` decided that the parent
       ordering was incorrect in the file graph.  (John Arbash Meinel)
 
+    * Some compatibility fixes for building the extensions with MSVC and
+      for python2.4. (John Arbash Meinel, #277484)
+
     * The index logic is now able to reload the list of pack files if and
       index ends up disappearing. We still don't reload if the pack data
       itself goes missing after checking the index. This bug appears as a

=== modified file 'bzrlib/_walkdirs_win32.pyx'
--- bzrlib/_walkdirs_win32.pyx	2008-09-26 05:14:51 +0000
+++ bzrlib/_walkdirs_win32.pyx	2008-10-23 18:26:30 +0000
@@ -81,7 +81,14 @@
     cdef readonly double st_ctime
     cdef readonly double st_mtime
     cdef readonly double st_atime
-    cdef readonly __int64 st_size
+    # We can't just declare this as 'readonly' because python2.4 doesn't define
+    # T_LONGLONG as a structure member. So instead we just use a property that
+    # will convert it correctly anyway.
+    cdef __int64 _st_size
+
+    property st_size:
+        def __get__(self):
+            return self._st_size
 
     # os.stat always returns 0, so we hard code it here
     cdef readonly int st_dev
@@ -180,7 +187,7 @@
         statvalue.st_ctime = _ftime_to_timestamp(&data.ftCreationTime)
         statvalue.st_mtime = _ftime_to_timestamp(&data.ftLastWriteTime)
         statvalue.st_atime = _ftime_to_timestamp(&data.ftLastAccessTime)
-        statvalue.st_size = _get_size(data)
+        statvalue._st_size = _get_size(data)
         statvalue.st_ino = 0
         statvalue.st_dev = 0
         return statvalue

=== modified file 'bzrlib/branch.py'
--- bzrlib/branch.py	2008-10-07 11:54:11 +0000
+++ bzrlib/branch.py	2008-10-22 19:07:07 +0000
@@ -1126,6 +1126,14 @@
         # (params) where params is a ChangeBranchTipParams with the members
         # (branch, old_revno, new_revno, old_revid, new_revid)
         self['post_change_branch_tip'] = []
+        # Introduced in 1.9
+        # Invoked when a stacked branch activates its fallback locations and
+        # allows the transformation of the url of said location.
+        # the api signature is
+        # (branch, url) where branch is the branch having its fallback
+        # location activated and url is the url for the fallback location.
+        # The hook should return a url.
+        self['transform_fallback_location'] = []
 
 
 # install the default hooks into the Branch class.
@@ -2021,6 +2029,13 @@
             errors.UnstackableBranchFormat):
             pass
         else:
+            for hook in Branch.hooks['transform_fallback_location']:
+                url = hook(self, url)
+                if url is None:
+                    hook_name = Branch.hooks.get_hook_name(hook)
+                    raise AssertionError(
+                        "'transform_fallback_location' hook %s returned "
+                        "None, not a URL." % hook_name)
             self._activate_fallback_location(url)
 
     def _check_stackable_repo(self):

=== modified file 'bzrlib/btree_index.py'
--- bzrlib/btree_index.py	2008-09-26 07:09:50 +0000
+++ bzrlib/btree_index.py	2008-10-15 21:34:10 +0000
@@ -139,6 +139,7 @@
         self._nodes = {}
         # Indicate it hasn't been built yet
         self._nodes_by_key = None
+        self._optimize_for_size = False
 
     def add_node(self, key, value, references=()):
         """Add a node to the index.
@@ -276,7 +277,8 @@
                     length = _PAGE_SIZE
                     if internal_row.nodes == 0:
                         length -= _RESERVED_HEADER_BYTES # padded
-                    internal_row.writer = chunk_writer.ChunkWriter(length, 0)
+                    internal_row.writer = chunk_writer.ChunkWriter(length, 0,
+                        optimize_for_size=self._optimize_for_size)
                     internal_row.writer.write(_INTERNAL_FLAG)
                     internal_row.writer.write(_INTERNAL_OFFSET +
                         str(rows[pos + 1].nodes) + "\n")
@@ -284,7 +286,8 @@
             length = _PAGE_SIZE
             if rows[-1].nodes == 0:
                 length -= _RESERVED_HEADER_BYTES # padded
-            rows[-1].writer = chunk_writer.ChunkWriter(length)
+            rows[-1].writer = chunk_writer.ChunkWriter(length,
+                optimize_for_size=self._optimize_for_size)
             rows[-1].writer.write(_LEAF_FLAG)
         if rows[-1].writer.write(line):
             # this key did not fit in the node:
@@ -313,7 +316,8 @@
                 # This will be padded, hence the -100
                 new_row.writer = chunk_writer.ChunkWriter(
                     _PAGE_SIZE - _RESERVED_HEADER_BYTES,
-                    reserved_bytes)
+                    reserved_bytes,
+                    optimize_for_size=self._optimize_for_size)
                 new_row.writer.write(_INTERNAL_FLAG)
                 new_row.writer.write(_INTERNAL_OFFSET +
                     str(rows[1].nodes - 1) + "\n")

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2008-10-12 16:11:17 +0000
+++ bzrlib/builtins.py	2008-10-17 08:32:52 +0000
@@ -3441,6 +3441,9 @@
             show_ids=False, verbose=False, this=False, other=False,
             include_merges=False):
         from bzrlib.missing import find_unmerged, iter_log_revisions
+        def message(s):
+            if not is_quiet():
+                self.outf.write(s)
 
         if this:
             mine_only = this
@@ -3464,7 +3467,7 @@
                                              " or specified.")
             display_url = urlutils.unescape_for_display(parent,
                                                         self.outf.encoding)
-            self.outf.write("Using saved parent location: "
+            message("Using saved parent location: "
                     + display_url + "\n")
 
         remote_branch = Branch.open(other_branch)
@@ -3488,8 +3491,8 @@
 
                 status_code = 0
                 if local_extra and not theirs_only:
-                    self.outf.write("You have %d extra revision(s):\n" %
-                                    len(local_extra))
+                    message("You have %d extra revision(s):\n" %
+                        len(local_extra))
                     for revision in iter_log_revisions(local_extra,
                                         local_branch.repository,
                                         verbose):
@@ -3501,9 +3504,9 @@
 
                 if remote_extra and not mine_only:
                     if printed_local is True:
-                        self.outf.write("\n\n\n")
-                    self.outf.write("You are missing %d revision(s):\n" %
-                                    len(remote_extra))
+                        message("\n\n\n")
+                    message("You are missing %d revision(s):\n" %
+                        len(remote_extra))
                     for revision in iter_log_revisions(remote_extra,
                                         remote_branch.repository,
                                         verbose):
@@ -3512,15 +3515,15 @@
 
                 if mine_only and not local_extra:
                     # We checked local, and found nothing extra
-                    self.outf.write('This branch is up to date.\n')
+                    message('This branch is up to date.\n')
                 elif theirs_only and not remote_extra:
                     # We checked remote, and found nothing extra
-                    self.outf.write('Other branch is up to date.\n')
+                    message('Other branch is up to date.\n')
                 elif not (mine_only or theirs_only or local_extra or
                           remote_extra):
                     # We checked both branches, and neither one had extra
                     # revisions
-                    self.outf.write("Branches are up to date.\n")
+                    message("Branches are up to date.\n")
             finally:
                 remote_branch.unlock()
         finally:

=== modified file 'bzrlib/chunk_writer.py'
--- bzrlib/chunk_writer.py	2008-08-28 20:13:31 +0000
+++ bzrlib/chunk_writer.py	2008-10-15 21:34:10 +0000
@@ -47,51 +47,53 @@
     #    In testing, some values for bzr.dev::
     #        repack  time  MB   max   full
     #         1       7.5  4.6  1140  0
-    #         2       8.4  4.2  1036  1          6.8
+    #         2       8.4  4.2  1036  1
     #         3       9.8  4.1  1012  278
     #         4      10.8  4.1  728   945
     #        20      11.1  4.1  0     1012
     #        repack = 0
-    #        zsync   time  MB    repack  max_z   time w/ add_node
-    #         0       6.7  24.7  0       6270    5.0
-    #         1       6.5  13.2  0       3342    4.3
-    #         2       6.6   9.6  0       2414    4.9
-    #         5       6.5   6.2  0       1549    4.8
-    #         6       6.5   5.8  1       1435    4.8
-    #         7       6.6   5.5  19      1337    4.8
-    #         8       6.7   5.3  81      1220    4.4
-    #        10       6.8   5.0  260     967     5.3
-    #        11       6.8   4.9  366     839     5.3
-    #        12       6.9   4.8  454     731     5.1
-    #        15       7.2   4.7  704     450     5.8
-    #        20       7.7   4.6  1133    7       5.8
+    #        zsync   time  MB    repack  stop_for_z
+    #         0       5.0  24.7  0       6270
+    #         1       4.3  13.2  0       3342
+    #         2       4.9   9.6  0       2414
+    #         5       4.8   6.2  0       1549
+    #         6       4.8   5.8  1       1435
+    #         7       4.8   5.5  19      1337
+    #         8       4.4   5.3  81      1220
+    #        10       5.3   5.0  260     967
+    #        11       5.3   4.9  366     839
+    #        12       5.1   4.8  454     731
+    #        15       5.8   4.7  704     450
+    #        20       5.8   4.6  1133    7
 
     #    In testing, some values for mysql-unpacked::
     #                next_bytes estim
-    #        repack  time  MB    hit_max full
-    #         1      51.7  15.4  3913  0
-    #         2      54.4  13.7  3467  0         35.4
-    #        20      67.0  13.4  0     3380      46.7
+    #        repack  time  MB    full    stop_for_repack
+    #         1            15.4  0       3913
+    #         2      35.4  13.7  0       346
+    #        20      46.7  13.4  3380    0
     #        repack=0
-    #        zsync                               time w/ add_node
-    #         0      47.7 116.5  0       29782   29.5
-    #         1      48.5  60.2  0       15356   27.8
-    #         2      48.1  42.4  0       10822   27.8
-    #         5      48.3  25.5  0       6491    26.8
-    #         6      48.0  23.2  13      5896    27.3
-    #         7      48.1  21.6  29      5451    27.5
-    #         8      48.1  20.3  52      5108    27.1
-    #        10      46.9  18.6  195     4526    29.4
-    #        11      48.8  18.0  421     4143    29.2
-    #        12      47.4  17.5  702     3738    28.0
-    #        15      49.6  16.5  1223    2969    28.9
-    #        20      48.9  15.7  2182    1810    29.6
-    #        30            15.4  3891    23      31.4
-
-    _max_repack = 0
-    _max_zsync = 8
-
-    def __init__(self, chunk_size, reserved=0):
+    #        zsync                       stop_for_z
+    #         0      29.5 116.5  0       29782
+    #         1      27.8  60.2  0       15356
+    #         2      27.8  42.4  0       10822
+    #         5      26.8  25.5  0       6491
+    #         6      27.3  23.2  13      5896
+    #         7      27.5  21.6  29      5451
+    #         8      27.1  20.3  52      5108
+    #        10      29.4  18.6  195     4526
+    #        11      29.2  18.0  421     4143
+    #        12      28.0  17.5  702     3738
+    #        15      28.9  16.5  1223    2969
+    #        20      29.6  15.7  2182    1810
+    #        30      31.4  15.4  3891    23
+
+    # Tuple of (num_repack_attempts, num_zsync_attempts)
+    # num_zsync_attempts only has meaning if num_repack_attempts is 0.
+    _repack_opts_for_speed = (0, 8)
+    _repack_opts_for_size = (20, 0)
+
+    def __init__(self, chunk_size, reserved=0, optimize_for_size=False):
         """Create a ChunkWriter to write chunk_size chunks.
 
         :param chunk_size: The total byte count to emit at the end of the
@@ -110,6 +112,8 @@
         self.num_zsync = 0
         self.unused_bytes = None
         self.reserved_size = reserved
+        # Default is to make building fast rather than compact
+        self.set_optimize(for_size=optimize_for_size)
 
     def finish(self):
         """Finish the chunk.
@@ -141,6 +145,19 @@
             self.bytes_list.append("\x00" * nulls_needed)
         return self.bytes_list, self.unused_bytes, nulls_needed
 
+    def set_optimize(self, for_size=True):
+        """Change how we optimize our writes.
+
+        :param for_size: If True, optimize for minimum space usage, otherwise
+            optimize for fastest writing speed.
+        :return: None
+        """
+        if for_size:
+            opts = ChunkWriter._repack_opts_for_size
+        else:
+            opts = ChunkWriter._repack_opts_for_speed
+        self._max_repack, self._max_zsync = opts
+
     def _recompress_all_bytes_in(self, extra_bytes=None):
         """Recompress the current bytes_in, and optionally more.
 

=== modified file 'bzrlib/help_topics/en/hooks.txt'
--- bzrlib/help_topics/en/hooks.txt	2008-09-23 23:45:04 +0000
+++ bzrlib/help_topics/en/hooks.txt	2008-10-22 19:05:22 +0000
@@ -174,6 +174,28 @@
 Please use the ``post_change_branch_tip`` hook instead.
 
 
+transform_fallback_location (Branch)
+------------------------------------
+
+Invoked as a stacked branch activates its fallback locations.
+
+The hook signature is (branch, url) where:
+
+  branch
+    The branch being opened.  Note that as it does not yet have its
+    fallback locations activated, the branch should be treated as
+    half-built.
+
+  url
+    The URL that the branch specified for its fallback location.
+
+The hook must return a URL for the branch containing the fallback
+location. If multiple hooks are registered, the order in which they 
+will be called is undefined and subject to change.
+
+(New in 1.9)
+
+
 server_started (SmartTCPServer)
 -------------------------------
 

=== modified file 'bzrlib/index.py'
--- bzrlib/index.py	2008-10-28 17:40:04 +0000
+++ bzrlib/index.py	2008-10-28 17:41:35 +0000
@@ -85,6 +85,7 @@
         self._nodes = {}
         self._nodes_by_key = None
         self._key_length = key_elements
+        self._optimize_for_size = False
 
     def _check_key(self, key):
         """Raise BadIndexKey if key is not a valid key for this index."""
@@ -279,6 +280,17 @@
                 (len(result.getvalue()), expected_bytes))
         return result
 
+    def set_optimize(self, for_size=True):
+        """Change how the builder tries to optimize the result.
+
+        :param for_size: Tell the builder to try and make the index as small as
+            possible.
+        :return: None
+        """
+        # GraphIndexBuilder itself doesn't pay attention to the flag yet, but
+        # other builders do.
+        self._optimize_for_size = for_size
+
 
 class GraphIndex(object):
     """An index for data with embedded graphs.

=== modified file 'bzrlib/plugin.py'
--- bzrlib/plugin.py	2008-10-07 06:41:46 +0000
+++ bzrlib/plugin.py	2008-10-27 07:29:32 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2004, 2005, 2007 Canonical Ltd
+# Copyright (C) 2004, 2005, 2007, 2008 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -33,7 +33,10 @@
 import os
 import sys
 
+from bzrlib import osutils
+
 from bzrlib.lazy_import import lazy_import
+
 lazy_import(globals(), """
 import imp
 import re
@@ -44,7 +47,6 @@
     config,
     debug,
     errors,
-    osutils,
     trace,
     )
 from bzrlib import plugins as _mod_plugins

=== modified file 'bzrlib/plugins/launchpad/account.py'
--- bzrlib/plugins/launchpad/account.py	2008-10-17 14:04:37 +0000
+++ bzrlib/plugins/launchpad/account.py	2008-10-23 17:43:37 +0000
@@ -54,14 +54,13 @@
     username = _config.get_user_option('launchpad_username')
     if username is not None:
         auth = AuthenticationConfig()
-        auth_usernames = _get_auth_user(auth)
-        for auth_username in auth_usernames.values():
-            if auth_username is not None and auth_username != username:
-                raise MismatchedUsernames()
+        auth_username = _get_auth_user(auth)
         # Auto-upgrading
-        if None in auth_usernames.values():
+        if auth_username is None:
             trace.note('Setting ssh/sftp usernames for launchpad.net.')
             _set_auth_user(username, auth)
+        elif auth_username != username:
+            raise MismatchedUsernames()
     return username
 
 
@@ -80,16 +79,13 @@
 def _get_auth_user(auth=None):
     if auth is None:
         auth = AuthenticationConfig()
-    return {'production': auth.get_user('ssh', 'bazaar.launchpad.net'),
-            'staging': auth.get_user('ssh', 'bazaar.staging.launchpad.net'),}
+    return auth.get_user('ssh', '.launchpad.net')
 
 def _set_auth_user(username, auth=None):
     if auth is None:
         auth = AuthenticationConfig()
     auth.set_credentials(
-        'Launchpad', 'bazaar.launchpad.net', username, 'ssh')
-    auth.set_credentials(
-        'Launchpad Staging', 'bazaar.staging.launchpad.net', username, 'ssh')
+        'Launchpad', '.launchpad.net', username, 'ssh')
 
 
 def check_lp_login(username, _transport=None):

=== modified file 'bzrlib/plugins/launchpad/test_account.py'
--- bzrlib/plugins/launchpad/test_account.py	2008-10-17 14:04:37 +0000
+++ bzrlib/plugins/launchpad/test_account.py	2008-10-23 17:43:37 +0000
@@ -64,21 +64,23 @@
                              'SSH keys with Launchpad.', str(error))
 
     def test_set_lp_login_updates_authentication_conf(self):
-        self.assertEqual([None, None], account._get_auth_user().values())
+        self.assertIs(None, account._get_auth_user())
         account.set_lp_login('foo')
-        self.assertEqual({'production': 'foo', 'staging': 'foo'},
-                         account._get_auth_user())
+        self.assertEqual('foo', account._get_auth_user())
 
     def test_get_lp_login_does_not_update_for_none_user(self):
         account.get_lp_login()
-        self.assertEqual([None, None], account._get_auth_user().values())
+        self.assertIs(None, account._get_auth_user())
 
     def test_get_lp_login_updates_authentication_conf(self):
         account._set_global_option('foo')
-        self.assertEqual([None, None], account._get_auth_user().values())
+        self.assertIs(None, account._get_auth_user())
         account.get_lp_login()
-        self.assertEqual({'production': 'foo', 'staging': 'foo'},
-                         account._get_auth_user())
+        auth = config.AuthenticationConfig()
+        self.assertEqual('foo', account._get_auth_user(auth))
+        self.assertEqual('foo', auth.get_user('ssh', 'bazaar.launchpad.net'))
+        self.assertEqual('foo', auth.get_user('ssh',
+                                              'bazaar.staging.launchpad.net'))
 
     def test_get_lp_login_leaves_existing_credentials(self):
         auth = config.AuthenticationConfig()

=== modified file 'bzrlib/python-compat.h'
--- bzrlib/python-compat.h	2008-09-26 05:14:51 +0000
+++ bzrlib/python-compat.h	2008-10-21 15:53:56 +0000
@@ -35,19 +35,33 @@
 #endif
 
 #if defined(_WIN32) || defined(WIN32)
-    /* Needed for htonl */
-    #include "Winsock.h"
-
     /* Defining WIN32_LEAN_AND_MEAN makes including windows quite a bit
      * lighter weight.
      */
     #define WIN32_LEAN_AND_MEAN
     #include <windows.h>
 
+    /* Needed for htonl */
+    #include "Winsock.h"
+
+    /* sys/stat.h doesn't have any of these macro definitions for MSVC, so
+     * we'll define whatever is missing that we actually use.
+     */
+    #if !defined(S_ISDIR)
+        #define S_ISDIR(m) (((m) & 0170000) == 0040000)
+    #endif
+    #if !defined(S_ISREG)
+        #define S_ISREG(m) (((m) & 0170000) == 0100000)
+    #endif
+    #if !defined(S_IXUSR)
+        #define S_IXUSR 0000100/* execute/search permission, owner */
+    #endif
     /* sys/stat.h doesn't have S_ISLNK on win32, so we fake it by just always
      * returning False
      */
-    #define S_ISLNK(mode) (0)
+    #if !defined(S_ISLNK)
+        #define S_ISLNK(mode) (0)
+    #endif
 #else /* Not win32 */
     /* For htonl */
     #include "arpa/inet.h"

=== modified file 'bzrlib/remote.py'
--- bzrlib/remote.py	2008-09-23 02:16:55 +0000
+++ bzrlib/remote.py	2008-10-27 06:14:45 +0000
@@ -799,7 +799,7 @@
         return repository.InterRepository.get(
             other, self).search_missing_revision_ids(revision_id, find_ghosts)
 
-    def fetch(self, source, revision_id=None, pb=None):
+    def fetch(self, source, revision_id=None, pb=None, find_ghosts=False):
         if self.has_same_location(source):
             # check that last_revision is in 'from' and then return a
             # no-operation.
@@ -807,9 +807,12 @@
                 not revision.is_null(revision_id)):
                 self.get_revision(revision_id)
             return 0, []
-        self._ensure_real()
-        return self._real_repository.fetch(
-            source, revision_id=revision_id, pb=pb)
+        inter = repository.InterRepository.get(source, self)
+        try:
+            return inter.fetch(revision_id=revision_id, pb=pb, find_ghosts=find_ghosts)
+
+        except NotImplementedError:
+            raise errors.IncompatibleRepositories(source, self)
 
     def create_bundle(self, target, base, fileobj, format=None):
         self._ensure_real()

=== modified file 'bzrlib/repofmt/pack_repo.py'
--- bzrlib/repofmt/pack_repo.py	2008-10-28 17:40:04 +0000
+++ bzrlib/repofmt/pack_repo.py	2008-10-28 17:41:35 +0000
@@ -977,6 +977,16 @@
         # TODO: combine requests in the same index that are in ascending order.
         return total, requests
 
+    def open_pack(self):
+        """Open a pack for the pack we are creating."""
+        new_pack = super(OptimisingPacker, self).open_pack()
+        # Turn on the optimization flags for all the index builders.
+        new_pack.revision_index.set_optimize(for_size=True)
+        new_pack.inventory_index.set_optimize(for_size=True)
+        new_pack.text_index.set_optimize(for_size=True)
+        new_pack.signature_index.set_optimize(for_size=True)
+        return new_pack
+
 
 class ReconcilePacker(Packer):
     """A packer which regenerates indices etc as it copies.
@@ -1475,6 +1485,7 @@
         self._names.pop(pack.name)
         self._packs_by_name.pop(pack.name)
         self._remove_pack_indices(pack)
+        self.packs.remove(pack)
 
     def _remove_pack_indices(self, pack):
         """Remove the indices for pack from the aggregated indices."""

=== modified file 'bzrlib/repository.py'
--- bzrlib/repository.py	2008-10-01 05:40:45 +0000
+++ bzrlib/repository.py	2008-10-27 06:15:46 +0000
@@ -2353,9 +2353,17 @@
     InterRepository.get(other).method_name(parameters).
     """
 
+    _walk_to_common_revisions_batch_size = 1
     _optimisers = []
     """The available optimised InterRepository types."""
 
+    def __init__(self, source, target):
+        InterObject.__init__(self, source, target)
+        # These two attributes may be overridden by e.g. InterOtherToRemote to
+        # provide a faster implementation.
+        self.target_get_graph = self.target.get_graph
+        self.target_get_parent_map = self.target.get_parent_map
+
     def copy_content(self, revision_id=None):
         raise NotImplementedError(self.copy_content)
 
@@ -2386,8 +2394,13 @@
         :param revision_ids: The start point for the search.
         :return: A set of revision ids.
         """
-        target_graph = self.target.get_graph()
+        target_graph = self.target_get_graph()
         revision_ids = frozenset(revision_ids)
+        # Fast path for the case where all the revisions are already in the
+        # target repo.
+        # (Although this does incur an extra round trip for the
+        # fairly common case where the target doesn't already have the revision
+        # we're pushing.)
         if set(target_graph.get_parent_map(revision_ids)) == revision_ids:
             return graph.SearchResult(revision_ids, set(), 0, set())
         missing_revs = set()
@@ -2395,25 +2408,40 @@
         # ensure we don't pay silly lookup costs.
         searcher = source_graph._make_breadth_first_searcher(revision_ids)
         null_set = frozenset([_mod_revision.NULL_REVISION])
+        searcher_exhausted = False
         while True:
-            try:
-                next_revs, ghosts = searcher.next_with_ghosts()
-            except StopIteration:
-                break
-            if revision_ids.intersection(ghosts):
-                absent_ids = set(revision_ids.intersection(ghosts))
-                # If all absent_ids are present in target, no error is needed.
-                absent_ids.difference_update(
-                    set(target_graph.get_parent_map(absent_ids)))
-                if absent_ids:
-                    raise errors.NoSuchRevision(self.source, absent_ids.pop())
-            # we don't care about other ghosts as we can't fetch them and
+            next_revs = set()
+            ghosts = set()
+            # Iterate the searcher until we have enough next_revs
+            while len(next_revs) < self._walk_to_common_revisions_batch_size:
+                try:
+                    next_revs_part, ghosts_part = searcher.next_with_ghosts()
+                    next_revs.update(next_revs_part)
+                    ghosts.update(ghosts_part)
+                except StopIteration:
+                    searcher_exhausted = True
+                    break
+            # If there are ghosts in the source graph, and the caller asked for
+            # them, make sure that they are present in the target.
+            # We don't care about other ghosts as we can't fetch them and
             # haven't been asked to.
-            next_revs = set(next_revs)
-            # we always have NULL_REVISION present.
-            have_revs = set(target_graph.get_parent_map(next_revs)).union(null_set)
-            missing_revs.update(next_revs - have_revs)
-            searcher.stop_searching_any(have_revs)
+            ghosts_to_check = set(revision_ids.intersection(ghosts))
+            revs_to_get = set(next_revs).union(ghosts_to_check)
+            if revs_to_get:
+                have_revs = set(target_graph.get_parent_map(revs_to_get))
+                # we always have NULL_REVISION present.
+                have_revs = have_revs.union(null_set)
+                # Check if the target is missing any ghosts we need.
+                ghosts_to_check.difference_update(have_revs)
+                if ghosts_to_check:
+                    # One of the caller's revision_ids is a ghost in both the
+                    # source and the target.
+                    raise errors.NoSuchRevision(
+                        self.source, ghosts_to_check.pop())
+                missing_revs.update(next_revs - have_revs)
+                searcher.stop_searching_any(have_revs)
+            if searcher_exhausted:
+                break
         return searcher.get_result()
    
     @deprecated_method(one_two)
@@ -2781,7 +2809,7 @@
             # till then:
             source_revision_ids = frozenset(self.source.all_revision_ids())
             revision_ids = source_revision_ids - \
-                frozenset(self.target.get_parent_map(source_revision_ids))
+                frozenset(self.target_get_parent_map(source_revision_ids))
             revision_keys = [(revid,) for revid in revision_ids]
             index = self.target._pack_collection.revision_index.combined_index
             present_revision_ids = set(item[1][0] for item in
@@ -2830,7 +2858,7 @@
         elif revision_id is not None:
             # Find ghosts: search for revisions pointing from one repository to
             # the other, and vice versa, anywhere in the history of revision_id.
-            graph = self.target.get_graph(other_repository=self.source)
+            graph = self.target_get_graph(other_repository=self.source)
             searcher = graph._make_breadth_first_searcher([revision_id])
             found_ids = set()
             while True:
@@ -2846,7 +2874,7 @@
             # Double query here: should be able to avoid this by changing the
             # graph api further.
             result_set = found_ids - frozenset(
-                self.target.get_parent_map(found_ids))
+                self.target_get_parent_map(found_ids))
         else:
             source_ids = self.source.all_revision_ids()
             # source_ids is the worst possible case we may need to pull.
@@ -3022,6 +3050,8 @@
 
 class InterOtherToRemote(InterRepository):
 
+    _walk_to_common_revisions_batch_size = 50
+
     def __init__(self, source, target):
         InterRepository.__init__(self, source, target)
         self._real_inter = None
@@ -3037,6 +3067,9 @@
             self.target._ensure_real()
             real_target = self.target._real_repository
             self._real_inter = InterRepository.get(self.source, real_target)
+            # Make _real_inter use the RemoteRepository for get_parent_map
+            self._real_inter.target_get_graph = self.target.get_graph
+            self._real_inter.target_get_parent_map = self.target.get_parent_map
     
     def copy_content(self, revision_id=None):
         self._ensure_real_inter()

=== modified file 'bzrlib/tests/__init__.py'
--- bzrlib/tests/__init__.py	2008-10-02 13:25:47 +0000
+++ bzrlib/tests/__init__.py	2008-10-17 00:50:41 +0000
@@ -1354,7 +1354,8 @@
         """
         # flush the log file, to get all content
         import bzrlib.trace
-        bzrlib.trace._trace_file.flush()
+        if bzrlib.trace._trace_file:
+            bzrlib.trace._trace_file.flush()
         if self._log_contents:
             # XXX: this can hardly contain the content flushed above --vila
             # 20080128

=== modified file 'bzrlib/tests/blackbox/test_missing.py'
--- bzrlib/tests/blackbox/test_missing.py	2008-08-02 17:10:50 +0000
+++ bzrlib/tests/blackbox/test_missing.py	2008-10-17 08:32:52 +0000
@@ -1,6 +1,4 @@
-# Copyright (C) 2005 Canonical Ltd
-# -*- coding: utf-8 -*-
-# vim: encoding=utf-8
+# Copyright (C) 2005, 2008 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -28,6 +26,21 @@
 
 class TestMissing(TestCaseWithTransport):
 
+    def test_missing_quiet(self):
+        # <https://bugs.launchpad.net/bzr/+bug/284748>
+        # create a source branch
+        #
+        # XXX: This still needs a test that missing is quiet when there are
+        # missing revisions.
+        a_tree = self.make_branch_and_tree('.')
+        self.build_tree_contents([('a', 'initial\n')])
+        a_tree.add('a')
+        a_tree.commit(message='initial')
+
+        out, err = self.run_bzr('missing -q .')
+        self.assertEqual('', out)
+        self.assertEqual('', err)
+
     def test_missing(self):
         missing = "You are missing 1 revision(s):"
 

=== modified file 'bzrlib/tests/branch_implementations/test_stacking.py'
--- bzrlib/tests/branch_implementations/test_stacking.py	2008-08-20 16:14:09 +0000
+++ bzrlib/tests/branch_implementations/test_stacking.py	2008-10-09 00:56:15 +0000
@@ -17,6 +17,7 @@
 """Tests for Branch.get_stacked_on_url and set_stacked_on_url."""
 
 from bzrlib import (
+    branch,
     bzrdir,
     errors,
     )
@@ -336,3 +337,23 @@
         rtree.lock_read()
         self.addCleanup(rtree.unlock)
         self.assertEqual('new content', rtree.get_file_by_path('a').read())
+
+    def test_transform_fallback_location_hook(self):
+        # The 'transform_fallback_location' branch hook allows us to inspect
+        # and transform the URL of the fallback location for the branch.
+        stack_on = self.make_branch('stack-on')
+        stacked = self.make_branch('stacked')
+        try:
+            stacked.set_stacked_on_url('../stack-on')
+        except (errors.UnstackableRepositoryFormat,
+                errors.UnstackableBranchFormat):
+            raise TestNotApplicable('Format does not support stacking.')
+        self.get_transport().rename('stack-on', 'new-stack-on')
+        hook_calls = []
+        def hook(stacked_branch, url):
+            hook_calls.append(url)
+            return '../new-stack-on'
+        branch.Branch.hooks.install_named_hook(
+            'transform_fallback_location', hook, None)
+        branch.Branch.open('stacked')
+        self.assertEqual(['../stack-on'], hook_calls)

=== modified file 'bzrlib/tests/test_btree_index.py'
--- bzrlib/tests/test_btree_index.py	2008-08-28 20:13:31 +0000
+++ bzrlib/tests/test_btree_index.py	2008-10-15 21:40:03 +0000
@@ -434,6 +434,13 @@
         self.assertEqual(sorted(nodes), nodes)
         self.assertEqual(16, len(nodes))
 
+    def test_set_optimize(self):
+        builder = btree_index.BTreeBuilder(key_elements=2, reference_lists=2)
+        builder.set_optimize(for_size=True)
+        self.assertTrue(builder._optimize_for_size)
+        builder.set_optimize(for_size=False)
+        self.assertFalse(builder._optimize_for_size)
+
     def test_spill_index_stress_2_2(self):
         # test that references and longer keys don't confuse things.
         builder = btree_index.BTreeBuilder(key_elements=2, reference_lists=2,

=== modified file 'bzrlib/tests/test_chunk_writer.py'
--- bzrlib/tests/test_chunk_writer.py	2008-08-22 05:54:44 +0000
+++ bzrlib/tests/test_chunk_writer.py	2008-10-15 21:34:10 +0000
@@ -39,6 +39,24 @@
         # Only a zlib header.
         self.assertEqual(4088, padding)
 
+    def test_optimize_for_speed(self):
+        writer = chunk_writer.ChunkWriter(4096)
+        writer.set_optimize(for_size=False)
+        self.assertEqual(chunk_writer.ChunkWriter._repack_opts_for_speed,
+                         (writer._max_repack, writer._max_zsync))
+        writer = chunk_writer.ChunkWriter(4096, optimize_for_size=False)
+        self.assertEqual(chunk_writer.ChunkWriter._repack_opts_for_speed,
+                         (writer._max_repack, writer._max_zsync))
+
+    def test_optimize_for_size(self):
+        writer = chunk_writer.ChunkWriter(4096)
+        writer.set_optimize(for_size=True)
+        self.assertEqual(chunk_writer.ChunkWriter._repack_opts_for_size,
+                         (writer._max_repack, writer._max_zsync))
+        writer = chunk_writer.ChunkWriter(4096, optimize_for_size=True)
+        self.assertEqual(chunk_writer.ChunkWriter._repack_opts_for_size,
+                         (writer._max_repack, writer._max_zsync))
+
     def test_some_data(self):
         writer = chunk_writer.ChunkWriter(4096)
         writer.write("foo bar baz quux\n")

=== modified file 'bzrlib/tests/test_index.py'
--- bzrlib/tests/test_index.py	2008-10-23 20:53:50 +0000
+++ bzrlib/tests/test_index.py	2008-10-28 17:41:35 +0000
@@ -350,6 +350,13 @@
         builder.add_node(('k', 'ey'), 'data', ([('reference', 'tokey')], ))
         builder.add_node(('reference', 'tokey'), 'data', ([],))
 
+    def test_set_optimize(self):
+        builder = GraphIndexBuilder(reference_lists=1, key_elements=2)
+        builder.set_optimize(for_size=True)
+        self.assertTrue(builder._optimize_for_size)
+        builder.set_optimize(for_size=False)
+        self.assertFalse(builder._optimize_for_size)
+
 
 class TestGraphIndex(TestCaseWithMemoryTransport):
 

=== modified file 'bzrlib/tests/test_repository.py'
--- bzrlib/tests/test_repository.py	2008-10-23 21:04:28 +0000
+++ bzrlib/tests/test_repository.py	2008-10-28 17:41:35 +0000
@@ -478,6 +478,12 @@
     def supports_rich_root(self):
         return False
 
+    def get_graph(self):
+        raise NotImplementedError
+
+    def get_parent_map(self, revision_ids):
+        raise NotImplementedError
+
 
 class InterDummy(repository.InterRepository):
     """An inter-repository optimised code path for DummyRepository.
@@ -1046,6 +1052,24 @@
     # thus there are not yet any tests.
 
 
+class TestOptimisingPacker(TestCaseWithTransport):
+    """Tests for the OptimisingPacker class."""
+
+    def get_pack_collection(self):
+        repo = self.make_repository('.')
+        return repo._pack_collection
+
+    def test_open_pack_will_optimise(self):
+        packer = pack_repo.OptimisingPacker(self.get_pack_collection(),
+                                            [], '.test')
+        new_pack = packer.open_pack()
+        self.assertIsInstance(new_pack, pack_repo.NewPack)
+        self.assertTrue(new_pack.revision_index._optimize_for_size)
+        self.assertTrue(new_pack.inventory_index._optimize_for_size)
+        self.assertTrue(new_pack.text_index._optimize_for_size)
+        self.assertTrue(new_pack.signature_index._optimize_for_size)
+
+
 class TestInterDifferingSerializer(TestCaseWithTransport):
 
     def test_progress_bar(self):

=== modified file 'bzrlib/transport/__init__.py'
--- bzrlib/transport/__init__.py	2008-10-06 06:34:36 +0000
+++ bzrlib/transport/__init__.py	2008-10-27 15:30:29 +0000
@@ -688,8 +688,21 @@
             # Now that we've read some data, see if we can yield anything back
             while cur_offset_and_size in data_map:
                 this_data = data_map.pop(cur_offset_and_size)
-                yield cur_offset_and_size[0], this_data
-                cur_offset_and_size = offset_stack.next()
+                this_offset = cur_offset_and_size[0]
+                try:
+                    cur_offset_and_size = offset_stack.next()
+                except StopIteration:
+                    # Close the file handle as there will be no more data
+                    # The handle would normally be cleaned up as this code goes
+                    # out of scope, but as we are a generator, not all code
+                    # will re-enter once we have consumed all the expected
+                    # data. For example:
+                    #   zip(range(len(requests)), readv(foo, requests))
+                    # Will stop because the range is done, and not run the
+                    # cleanup code for the readv().
+                    fp.close()
+                    cur_offset_and_size = None
+                yield this_offset, this_data
 
     def _sort_expand_and_combine(self, offsets, upper_limit):
         """Helper for readv.

=== modified file 'bzrlib/win32utils.py'
--- bzrlib/win32utils.py	2008-10-01 05:40:45 +0000
+++ bzrlib/win32utils.py	2008-10-27 07:29:32 +0000
@@ -319,7 +319,9 @@
 
 
 def _ensure_unicode(s):
+    from bzrlib import osutils
     if s and type(s) != unicode:
+        from bzrlib import osutils
         s = s.decode(osutils.get_user_encoding())
     return s
 

=== modified file 'doc/en/user-guide/branching_a_project.txt'
--- doc/en/user-guide/branching_a_project.txt	2008-05-29 18:42:07 +0000
+++ doc/en/user-guide/branching_a_project.txt	2008-10-16 10:57:16 +0000
@@ -37,7 +37,9 @@
 where to put it on your filesystem. For maximum storage
 efficiency down the track, it is recommended that branches
 be created somewhere under a directory that has been set up
-as a shared repository. For example::
+as a shared repository. (See `Choosing a shared repository
+layout`_ in the appendices for guidelines and examples.)
+For example::
 
   bzr init-repo my-repo
   cd my-repo
@@ -89,7 +91,8 @@
 the latest version.
 
 As explained in later chapters, Bazaar also has support for
-lightweight checkouts of a branch, i.e. working trees with
+`lightweight checkouts <#getting-a-lightweight-checkout>`_ 
+of a branch, i.e. working trees with
 no local storage of history. Of course, disconnected usage
 is not available then but that's a tradeoff you can decide
 to make if local disk space is really tight for you. Support for

=== modified file 'doc/en/user-guide/core_concepts.txt'
--- doc/en/user-guide/core_concepts.txt	2008-05-09 14:59:31 +0000
+++ doc/en/user-guide/core_concepts.txt	2008-10-16 10:57:16 +0000
@@ -108,11 +108,15 @@
 to use a *standalone tree*, which has a working tree, branch, and repository
 all in a single location. Other common scenarios include:
 
-* *Shared repositories* - working tree and branch are colocated, but the
-  repository is in a higher level directory.
-
-* *Lightweight checkouts* - branch is stored is a different location
-  to the working tree.
+* `Shared repositories <#a-reminder-about-shared-repositories>`_ - working
+  tree and branch are colocated, but the repository is in a higher level
+  directory.
+
+* `Stacked branches <#using-stacked-branches>`_ - branch stores just its
+  unique revisions, using its parent's repository for common revisions.
+
+* `Lightweight checkouts <#getting-a-lightweight-checkout>`_ - branch is 
+  stored in a different location to the working tree.
 
 The best way to use Bazaar, however, depends on your needs. Let's take a
 look at some common workflows next.

=== modified file 'doc/en/user-guide/using_checkouts.txt'
--- doc/en/user-guide/using_checkouts.txt	2007-11-23 05:52:03 +0000
+++ doc/en/user-guide/using_checkouts.txt	2008-10-16 10:57:16 +0000
@@ -61,10 +61,13 @@
 Of course, many of the benefits of a normal checkout are lost by doing
 this but that's a tradeoff you can make if and when it makes sense.
 
+The ``--lightweight`` option only applies to checkouts, not to all branches.
+
 Note: If your code base is really large and disk space on your computer
 is limited, lightweight checkouts may be the right choice for you.
-Be sure to consider all your options though including shared repositories
-and `Reusing a checkout`_ as explained later.
+Be sure to consider all your options though including
+`shared repositories <#a-reminder-about-shared-repositories>`_,
+`stacked branches <#using-stacked-branches>`_, and `reusing a checkout`_.
 
 Updating to the latest content
 ------------------------------

